@@ -133,7 +133,6 @@ double cv_func_regression_categorical_ls(double *vector_scale_factor){
         return(DBL_MAX);
     }
 
-/* Compute the cross-validation function */
     if((BANDWIDTH_reg_extern == BW_FIXED)||(int_ll_extern == LL_LC)){ 
       return(np_kernel_estimate_regression_categorical_ls_aic(
         int_ll_extern,
@@ -152,9 +151,9 @@ double cv_func_regression_categorical_ls(double *vector_scale_factor){
         vector_Y_extern,
         &vector_scale_factor[1],
         num_categories_extern));
-        } else {
+      } else {
       return(cv_func_regression_categorical_ls_nn(vector_scale_factor));
-          }
+      }
 }
 
 double cv_func_regression_categorical_ls_nn(double *vector_scale_factor)
@@ -672,10 +671,6 @@ double cv_func_regression_categorical_aic_c(double *vector_scale_factor)
         return(DBL_MAX);
     }
 
-/* Compute the AIC_c function */
-/* The local linear component of Tristen's efficient code is broken
-   under MPI, so we comment it out for the MPI code here pending a
-   fix. Also, bwscaling and cv.aic break this code. */
     if((BANDWIDTH_reg_extern == BW_FIXED)||(int_ll_extern == LL_LC)){
       return(np_kernel_estimate_regression_categorical_ls_aic(
         int_ll_extern,
@@ -694,7 +689,7 @@ double cv_func_regression_categorical_aic_c(double *vector_scale_factor)
         vector_Y_extern,
         &vector_scale_factor[1],
         num_categories_extern));
-    } else {
+        } else {
       return(kernel_estimate_regression_categorical_aic_c(
         int_ll_extern,
         KERNEL_reg_extern,
